//importing functions and objects from D3 library
import {
  select,
  csv,
  scaleLinear,
  scaleTime,
  extent,
  max,
  axisLeft,
  axisBottom,
  area,
  curveBasis,
  format
} from 'd3';

// svg element and storing values in width and height variables
const svg = select('svg');
const width = +svg.attr('width');
const height = +svg.attr('height');

//displaying the title
const render = data => {
  const title = 'Female Suicides from 1985-2015';
  //x-axis label
  const xValue = d => d.year;
  const xAxisLabel = 'Years';
  //y-axis label
  const yValue = d => d.suicides_no;
  const circleRadius = 6;
  const yAxisLabel = 'No. of Suicides';
  //styles of chart
  const margin = { top: 60, right: 40, bottom: 88, left: 105 };
  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;
  
  //map the data values to the x and y coordinates on the chart
  const xScale = scaleTime()
    .domain(extent(data, xValue))
    .range([0, innerWidth])
    .nice();
  
  const yScale = scaleLinear()
    .domain([0, max(data, yValue)])
    .range([innerHeight, 0])
    .nice();
  
  const g = svg.append('g')
    .attr('transform', `translate(${margin.left},${margin.top})`);
  
  const xAxis = axisBottom(xScale)
    .ticks(6)
    .tickSize(-innerHeight)
    .tickPadding(15);

  const yAxisTickFormat = number =>
    format('.1s')(number)
      .replace('G', 'B');
  
  const yAxis = axisLeft(yScale)
    .tickSize(-innerWidth)
    .tickPadding(10)
    .tickFormat(yAxisTickFormat);
  
  const yAxisG = g.append('g').call(yAxis);
  yAxisG.selectAll('.domain').remove();
  // defining attributes and styles for y-axis label
  yAxisG.append('text')
      .attr('class', 'axis-label')
      .attr('y', -60) // specifying y position for the label
      .attr('x', -innerHeight / 2) // specifying x position for the label
      .attr('transform', `rotate(-90)`) // rotating the label by -90 degrees
      .attr('text-anchor', 'middle') // aligning the label in middle
      .text(yAxisLabel);
  
  const xAxisG = g.append('g').call(xAxis)
    .attr('transform', `translate(0,${innerHeight})`);
  
  xAxisG.select('.domain').remove();
  // defining attributes and styles for x-axis label
  xAxisG.append('text')
      .attr('class', 'axis-label')
      .attr('y', 50) // specifying y position for the label
      .attr('x', innerWidth / 2)  // specifying x position for the label
      .text(xAxisLabel);
  //area graph generator
  const areaGenerator = area()
    .x(d => xScale(xValue(d)))
    .y0(innerHeight)
    .y1(d => yScale(yValue(d)))
    .curve(curveBasis);
  
  g.append('path')
      .attr('class', 'line-path')
      .attr('d', areaGenerator(data));
  
  svg.append('text')
      .attr('class', 'title')
      .attr('x', width / 2)
      .attr('y', 45)
      .text(title);
};
//gist URL of my dataset
//load the data from a CSV file 
csv('https://gist.githubusercontent.com/yamunayennem/2b33182daebe91db20a7b1ec174f85d0/raw/9510498318bc5ef54448dfdff53bd8f7b2345aa9/deaths_female.csv')
  .then(data => {
    data.forEach(d => {
      d.suicides_no = +d.suicides_no;
      d.year = new Date(d.year);
    });
    render(data); // render the area chart by calling the render function with the loaded data
  });